@@ -11,7 +11,7 @@ class TestGemSpecFetcher < Gem::TestCase
   def setup
     super
 
-    @uri = URI.parse @gem_repo
+    @uri = Gem::URI.parse @gem_repo
     @source = Gem::Source.new(@uri)
 
     @sf = Gem::SpecFetcher.new
@@ -109,9 +109,7 @@ class TestGemSpecFetcher < Gem::TestCase
   def test_spec_for_dependency_platform
     util_set_arch "i386-linux"
 
-    spec_fetcher do |fetcher|
-      fetcher.legacy_platform
-    end
+    spec_fetcher(&:legacy_platform)
 
     dep = Gem::Dependency.new "pl", 1
     specs_and_sources, _ = @sf.spec_for_dependency dep
@@ -127,9 +125,7 @@ class TestGemSpecFetcher < Gem::TestCase
   def test_spec_for_dependency_mismatched_platform
     util_set_arch "hrpa-989"
 
-    spec_fetcher do |fetcher|
-      fetcher.legacy_platform
-    end
+    spec_fetcher(&:legacy_platform)
 
     dep = Gem::Dependency.new "pl", 1
     specs_and_sources, errors = @sf.spec_for_dependency dep
@@ -173,19 +169,24 @@ class TestGemSpecFetcher < Gem::TestCase
     spec_fetcher do|fetcher|
       fetcher.spec "example", 1
       fetcher.spec "other-example", 1
+      fetcher.spec "an-example", 1
       fetcher.spec "examp", 1
     end
 
     suggestions = @sf.suggest_gems_from_name("examplw", :latest, 1)
     assert_equal ["example"], suggestions
 
-    suggestions = @sf.suggest_gems_from_name("other")
-    assert_equal ["other-example"], suggestions
+    suggestions = @sf.suggest_gems_from_name("anexample")
+    assert_equal ["an-example"], suggestions
 
-    suggestions = @sf.suggest_gems_from_name("exam")
-    assert suggestions.any? { ["examp"] }
-    assert suggestions.any? { ["example"] }
-    assert suggestions.any? { ["other-example"] }
+    suggestions = @sf.suggest_gems_from_name("xample")
+    assert_equal ["example"], suggestions
+
+    suggestions = @sf.suggest_gems_from_name("amp")
+    assert_equal [], suggestions
+
+    suggestions = @sf.suggest_gems_from_name("other-apple")
+    assert_equal ["other-example"], suggestions
   end
 
   def test_suggest_gems_from_name_prerelease
