@@ -2,5 +2,5 @@
 
 source "https://rubygems.org"
 
-gem "rubocop", "~> 1.30"
+gem "rubocop", ">= 1.52.1", "< 2"
 gem "rubocop-performance", "~> 1.14"
